Attribute VB_Name = "modMain"
Option Explicit

'Log Constants
'Private Const VbLogAuto = 0&         ' If running on Windows 95, this option logs messages to the file specified in the LogFile property. If running on Windows NT, messages are logged to the NT Application Event Log, with the App.Title string used as the application source.
Private Const VbLogOff = 1&          ' Turns all logging off. Messages from UI shunts as well as from the LogEvent method are ignored and discarded.
Private Const VbLogToFile = 2&       ' Forces logging to a file. If no valid filename is present in LogPath, logging is ignored, and the property is set to vbLogOff.
Private Const VbLogToNT = 3&         ' Forces logging to the NT event log. If not running on Windows NT, or the event log is unavailable, logging is ignored and the property is set to vbLogOff.
'Private Const VbLogOverwrite = &H10& ' Indicates that the logfile should be recreated each time the application starts. This value can be combined with other mode options using the OR operator. The default action for logging is to append to the existing file. In the case of NT event logging, this flag has no meaning.
'Private Const VbLogThreadID = &H20&  ' Indicates that the current thread ID be prepended to the message, in the form "[T:0nnn] ". This value can be combined with other mode options using the OR operator. The default action is to show the thread ID only when the application is multi-threaded (either explicitly marked as thread-safe, or implemented as an implicit multithreaded app, such as a local server with the instancing property set to Single-Use, multithreaded).
Private lngLogMode  As Long
Private strLogFile  As String
Private bLog        As Boolean


Private Type COPYDATASTRUCT
        dwData As Long
        cbData As Long
        lpData As Long
End Type

' Declare necessary API routines:
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (hpvDest As Any, hpvSource As Any, ByVal cbCopy As Long)
Private Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hwnd As Long, ByVal Msg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hwnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long
Private Declare Function FindWindow Lib "user32" Alias "FindWindowA" (ByVal lpClassName As Any, ByVal lpWindowName As Any) As Long
Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
                
Private Const WM_COPYDATA = &H4A
Private Const GWL_WNDPROC = (-4)
Private hHandOrig   As Long

Public Sub SetLogMode()
    'get Log Active
    bLog = CBool(GetSetting("JetSoft", App.EXEName, "LogActive", "False"))

    If bLog Then
        'Get Settings
        strLogFile = GetSetting("JetSoft", App.EXEName, "LogFile", "")
        lngLogMode = IIf(strLogFile = "", VbLogToNT, VbLogToFile)
    End If
End Sub

Public Sub LogMsg(strAction$, strMessage$, lngType As LogEventTypeConstants)
    'Only process if logging enabled
    If bLog Then
        'Note: start and stop logging each time to prevent locking log file
        App.StartLogging strLogFile, lngLogMode
        App.LogEvent Left(strAction$ & ":              ", 15) & Now & " " & strMessage$, lngType
        App.StartLogging strLogFile, VbLogOff
    End If
End Sub

Public Sub Main()
On Error GoTo hard_error
    Dim hWndMonitor As Long
    Dim lngQueue    As Long
    Dim buf()       As Byte
    Dim cds         As COPYDATASTRUCT
    Dim strQueue    As String
   
    'Start logging
    SetLogMode
   
    'Determine Command Line options
    strQueue = Left(Command$, 255)
    LogMsg "Add", strQueue, vbLogEventTypeInformation

    If App.PrevInstance Then
        'Copy String to Byte Array
        lngQueue = Len(strQueue)
        buf = StrConv(strQueue, vbFromUnicode)
        
        'Set Up send structure
        cds.dwData = 3
        cds.cbData = Len(strQueue) + 1
        cds.lpData = VarPtr(buf(0))
                 
        'Pass call to previous instance
        hWndMonitor = FindWindow(0&, App.EXEName & " - Queue Monitor")
        If hWndMonitor = 0 Then
            If Not CBool(GetSetting("JetSoft", App.EXEName, "LogActive", "False")) Then MsgBox "No Window"
            LogMsg "Queue Not Found", strQueue, vbLogEventTypeError
            Exit Sub
        End If
        SendMessage hWndMonitor, WM_COPYDATA, 0, cds
    Else
        'Load Form
        LogMsg "Start Queue", "", vbLogEventTypeInformation
        frmQueue.Show
        
        'Add if any options specified
        If strQueue <> "" Then frmQueue.AddQueue strQueue
    End If
Exit Sub
hard_error:
    LogMsg "Launch Error", Err.Number & ":" & Err.Description, vbLogEventTypeError
End Sub

Public Function HookAdd()
    hHandOrig = SetWindowLong(frmQueue.hwnd, GWL_WNDPROC, AddressOf HookHandle)
End Function

Public Function HookHandle(ByVal hwnd As Long, ByVal lngMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
    Dim buf()   As Byte
    Dim cds     As COPYDATASTRUCT
    Dim strQueue  As String
    
    If lngMsg = WM_COPYDATA Then
        'Retrieve info
        Call CopyMemory(cds, ByVal lParam, Len(cds))
        ReDim buf(cds.cbData)
        Call CopyMemory(buf(0), ByVal cds.lpData, cds.cbData)
        strQueue = StrConv(buf, vbUnicode)
        strQueue = Left$(strQueue, InStr(1, strQueue, Chr$(0)) - 1)

        'Add to list
        frmQueue.AddQueue strQueue
        
    Else
        HookHandle = CallWindowProc(hHandOrig, hwnd, lngMsg, wParam, lParam)
    End If
    
End Function

Public Function HookRemove()
    SetWindowLong frmQueue.hwnd, GWL_WNDPROC, hHandOrig
End Function

